unit Nodes;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, StdCtrls,
	Node, Emitter, Stack, Heap;


type

	TSNode = class(TNode);

	TStmtListNode = class(TNode);

	TStmtNode = class(TNode);

	TAssignmentStmtNode = class(TNode)
	public
		procedure CodeGen; override;
	end;

	TAddExprNode = class(TNode)
	public
		procedure CodeGen; override;
	end;

	TSubExprNode = class(TNode)
	public
		procedure CodeGen; override;
	end;

	TTimesExprNode = class(TNode)
	public
		procedure CodeGen; override;
	end;

	TDivideExprNode = class(TNode)
	public
		procedure CodeGen; override;
	end;


	TSimpleExprNode = class(TNode);

	TShowStmtNode = class(TNode)
	public
		procedure CodeGen; override;
	end;

	TIdentifierNode = class(TNode)
	public
		sVal: String;
		constructor Create(sValue: String);
		procedure CodeGen; override;
	end;

	TNumberNode = class(TNode)
	public
		iVal: Integer;
		constructor Create(iValue: Integer);
		procedure CodeGen; override;
	end;



// ==========================================================
implementation

{ TIdentifierNode }

// -----------------------------------------------------------
// identifier constructor
constructor TIdentifierNode.Create(sValue: String);
begin
	sVal := sValue;
end;

// -----------------------------------------------------------
// identifier
procedure TIdentifierNode.CodeGen;
begin
	inherited;
	stak.Push(heep.GetVal(sVal));
end;


{ TNumberNode }

// -----------------------------------------------------------
// number constructor
constructor TNumberNode.Create(iValue: Integer);
begin
	iVal := iValue;
end;


// -----------------------------------------------------------
// number codegen
procedure TNumberNode.CodeGen;
begin
	inherited;
	stak.Push(iVal);
end;


{ TAssignmentStmtNode }

// -----------------------------------------------------------
// assignment
procedure TAssignmentStmtNode.CodeGen;
begin
	inherited;
	heep.SetVal(TIdentifierNode(nodeLeft).sVal, stak.Pop);
end;

{ TShowStmtNode }

// -----------------------------------------------------------
// show statement
procedure TShowStmtNode.CodeGen;
begin
	inherited;
	emit.Emit('The value of ' + TIdentifierNode(nodeLeft).sVal + ' is ' +
		IntToStr(heep.GetVal(TIdentifierNode(nodeLeft).sVal)));
end;

{ TAddExprNode }

// -----------------------------------------------------------
// add operation
procedure TAddExprNode.CodeGen;
begin
	inherited;
	if Assigned(nodeRight) then
		stak.Push(stak.Pop + stak.Pop);
end;

{ TSubExprNode }

// -----------------------------------------------------------
// subtract operation
procedure TSubExprNode.CodeGen;
var
	iTop: Integer;
begin
	inherited;
	if Assigned(nodeRight) then
	begin
		iTop := stak.Pop;
		stak.Push(stak.Pop - iTop);
	end;

end;

{ TTimesExprNode }

// -----------------------------------------------------------
// times operation
procedure TTimesExprNode.CodeGen;
begin
	inherited;
	if Assigned(nodeRight) then
		stak.Push(stak.Pop * stak.Pop);
end;

{ TDivideExprNode }

// -----------------------------------------------------------
// divice operation
procedure TDivideExprNode.CodeGen;
var
	iTop: Integer;
begin
	inherited;
	if Assigned(nodeRight) then
	begin
		iTop := stak.Pop;
		stak.Push(stak.Pop div iTop); // note we are doing integer division here
	end;

end;

end.
